/*
 * EA878 - Lab. Micro e Mini: Software
 *
 * Processamento do protocolo HTTP 1.1 com lex e yacc
 *
 * Eleri Cardozo, Marco A. A. Henriques
 */


/*
 *  HTTP PDU format:
 *
 *  <request> /<resource> HTTP/1.1
 *  Host: <values>
 *  <parameter>: <values>
 *  ...
 *  <parameter>: <values>
 *  newline
 */


/*
 * Interaction with lex:
 *
 * yylex returns the type of token in a http pdu
 *
 */


%{
#include <stdio.h>
char value[1024];
char resource[1024];
int varindex = 0;
int have_host = 0;   /* Host parameter is mandatory for HTTP 1.1 */
%}

%union {
   char text;
 }

%token NL CR COMMA END SLASH BLANK
%token GET HEAD OPTIONS TRACE POST PUT DELETE VERSION
%token VERSION11
%token CONNECTION USER_AGENT HOST ACCEPT ACCEPT_ENCODING ACCEPT_LANGUAGE ACCEPT_CHARSET KEEP_ALIVE IF_MODIFIED_SINCE CACHE_CONTROL REFERER TE AUTHORIZATION COOKIE PRAGMA
%token <text> CHARACTER

%%
http_message: request parameters;    /* what an HTTP message looks like */

request: reqtype resource version CR NL
       | reqtype resource version NL
       ;     /* the first line of an HTTP message */

reqtype: GET          {printf("Request: GET\n");}
       | HEAD         {}
       | OPTIONS      {}
       | TRACE        {} 
       | POST         {} 
       | PUT          {}
       | DELETE       {}
       | VERSION      {printf("Request: VERSION\n");}
       ;


resource: SLASH chars;    /* /arq.html */

chars: /* nothing */
     | chars char
     | chars slash char
     ;

slash: SLASH {value[varindex++] = '/';}

char: CHARACTER {value[varindex++] = yyval.text;}
    | BLANK {print_resource();}

version: VERSION11 {printf("\n\nVersion: 1.1");}

parameters: /* no parameter */
         | parameters parameter
         ;

parameter: partype fields   /* remaining parameters */
         ;

partype: CONNECTION        {printf("\n\nConnection parameters:\n");}
       | USER_AGENT        {}
       | ACCEPT            {}
       | ACCEPT_ENCODING   {}
       | ACCEPT_LANGUAGE   {}
       | ACCEPT_CHARSET    {}
       | KEEP_ALIVE        {}
       | IF_MODIFIED_SINCE {}
       | CACHE_CONTROL     {}
       | REFERER           {}
       | TE                {}
       | AUTHORIZATION     {}
       | COOKIE            {}
       | PRAGMA            {}
       | HOST              {printf("\n\nHost parameters:\n"); have_host = 1;}
       ;


fields:  /* nothing */
      | fields field
      ;


field: CHARACTER                 /* a char belonging to a value */
               {if(! (varindex == 0 && yyval.text == ' '))
                  value[varindex++] = yyval.text;}
      | SLASH               /* slash - treat as character */
               {value[varindex++] = '/';}
      | BLANK               /* blank - treat as character */
               {value[varindex++] = ' ';}
      | COMMA               /* end of value */
               {print_value();}
      | NL                  /* end of value */
               {print_value();}
      | CR                  /* carriage return -- do nothing */
               {}
      | END                 /* end of parsing */
               {print_value();
               if(have_host == 0) yyerror("No HOST partameter");   /* host parameter absent */
               return(0);}
      | error {yyerror;}    /* if none above, report error */
      ;

%%


int lineno = 0;
extern char *yytext;


print_resource()
{
  int i,j=0;
  char *temp;

  temp = (char *) strdup(value);
  
  for(i = 0; i < varindex; i++) value[i + 1] = temp[j++];  /* shift */
  value[0] = '/';                  /* insert / */
  value[varindex+1] = '\0';             /* terminate string */
  varindex = 0;                       /* reset varindex */
  printf("\nResource: %s", value); /* print value */
  free(temp);
}

print_value()
  {
    int i;
 
    if(varindex == 0) return(0);        /* no value */
    value[varindex] = '\0';             /* terminate string */
    varindex = 0;                       /* reset varindex */

    while(value[0]==' ') { /* shift */
      i=0;
      do {
	value[i]=value[i+1];
	i++;
      }
      while(value[i]!='\0');
    }
    
    printf("\nValue: %s", value);    /* print value */
  }



yyerror(s)
     char *s;
 {
   printf("\nhttp parser error: %s", s);
   printf("\nline %d", lineno);
   printf("\nlast token read: |%s|\n", yytext);
 }
